﻿BUF_Area = {}

function BUF_Area:new()
	local o = {
		num = 0,    -- Anzahl Frames
		frame = {}, -- Array von Unitframes

		-- lookup-Tables
		unitId = {}, -- UnitID => index
		vehicleId = {}, -- Vehicle UnitID => index
		
		MaxColumns = 5,
		MaxRows = 8,
		MinDisplayColumns = 5,
		MinDisplayRows = 4,
		Spacer = 2,
		Height = 0,
		Width = 0,
		Enabled = false, -- Grundsätzlich aktiviert
		Active = true, -- aktuell aktiviert
		Movable = false,
		Name = "",
		Horizontal = false, -- Gruppen horizontal statt vertikal anordnen
		Template = "standard",
		LateCreation = true, -- GUI Elemente erst bei ersten Verwendung erzeugen
		Targets = false, -- die 2. Hälfte der Frames sind Targets		
		
		ufWidth = 0,
		ufHeight = 0,

		PendingUpdate = false, 
	}
	setmetatable(o, self)
	self.__index = self

	o:Create()
	return o
end

-- empty stub
function BUF_Area:Create()
end

function BUF_Area:ApplyOptions(options)
	local areaOpt = nil
	if options then
		areaOpt = options[self.Name]
	end
	if areaOpt then
		self.Enabled = areaOpt.Enabled
		self.Horizontal = areaOpt.Horizontal
		if (areaOpt.Height or 0) > 0 then
			self.Height = areaOpt.Height
		end
		if (areaOpt.Width or 0) > 0 then
			self.Width = areaOpt.Width
		end
		if (areaOpt.MaxColumns or 0) > 0 then
			self.MaxColumns = areaOpt.MaxColumns
		else
			areaOpt.MaxColumns = self.MaxColumns
		end
		if (areaOpt.MaxRows or 0) > 0 then
			self.MaxRows = areaOpt.MaxRows
		else
			areaOpt.MaxRows = self.MaxRows
		end
		if (areaOpt.MinDisplayRows or 0) > 0 then
			self.MinDisplayRows = areaOpt.MinDisplayRows
		else
			areaOpt.MinDisplayRows = self.MinDisplayRows
		end
		if (areaOpt.Spacer or 0) > 0 then
			self.Spacer = areaOpt.Spacer
		else
			areaOpt.Spacer = self.Spacer
		end
		if areaOpt.Template then
			self.Template = areaOpt.Template
		else
			areaOpt.Template = self.Template
		end
	end
	return areaOpt
end

function BUF_Area:CreateGUI(options)
	local firstFrame, relativeFrame, relativePoint, offx, offy
		
	-- Frame für Area erzeugen
	self.BaseFrame = CreateFrame("Frame", "BUF_" .. self.Name, UIParent)	
	self.BaseFrame.area = self
	
	local areaOpt = self:ApplyOptions(options)
		
	if areaOpt and areaOpt.point then
		self.BaseFrame:SetPoint(areaOpt.point, UIParent, areaOpt.relativePoint, areaOpt.x, areaOpt.y)
	else
		self.BaseFrame:SetPoint("CENTER", 0 ,0)
	end		
	self.BaseFrame:SetHeight(self.Height)
	self.BaseFrame:SetWidth(self.Width)	
	
	-- Create resizing corner grip
	self.Grip = CreateFrame("Button", "BUF_" .. self.Name .. "_ResizeGrip", self.BaseFrame)
	self.Grip.area = self
	self.Grip:SetNormalTexture("Interface\\AddOns\\BakorsUF\\images\\ResizeGrip")
	self.Grip:SetHighlightTexture("Interface\\AddOns\\BakorsUF\\images\\ResizeGrip")
	self.Grip:SetWidth(16)
	self.Grip:SetHeight(16)
	self.Grip:SetPoint("BOTTOMRIGHT", self.BaseFrame, "BOTTOMRIGHT", 0, 1)
	self.Grip:SetScript("OnMouseDown", function(this, button) this.area.BaseFrame:StartSizing() end)
	self.Grip:SetScript("OnMouseUp", function(this) BUF_Roster:GripMouseUp(this.area) end)
	self.Grip:Hide()
	
	self.Label = self.BaseFrame:CreateFontString(nil, nil, "GameFontNormal")
	self.Label:SetAllPoints(self.BaseFrame)
	self.Label:SetText(self.Name)
	self.Label:Hide()
		
	self.BaseFrame.TimeSinceLastUpdate = 0
	self.BaseFrame:SetScript("OnUpdate",
		function(frame, elapsed)
			frame.TimeSinceLastUpdate = frame.TimeSinceLastUpdate + elapsed 		

			if frame.TimeSinceLastUpdate > 1 then
				frame.TimeSinceLastUpdate = 0

				if not frame:IsMovable() then frame.area:UpdateAllUnitFrames() end
			end
			if frame.area.PendingUpdate then
				frame.area:UpdateAllUnitFrames()
				frame.area.PendingUpdate = false
			end
		end
	)
	
	self.BaseFrame:SetScript("OnDragStart", function(this) if(this:IsMovable()) then this:StartMoving() end end)
	self.BaseFrame:SetScript("OnDragStop", function(this) BUF_Roster:DragStop(this) end)
	self.BaseFrame:RegisterForDrag("LeftButton")
	
	self:Rearrange(true)
end

function BUF_Area:EnableMove()
	if self.Enabled then
		self.BaseFrame:SetBackdrop(
			{bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
			edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
			tile = true, tileSize = 16, edgeSize = 16, 
			insets = { left = 4, right = 4, top = 4, bottom = 4 }})
		self.BaseFrame:SetBackdropColor(0, 0, 0, 1)
		self.BaseFrame:Show()
		self.Grip:Show()
		self.Label:Show()
		
		for i = 1, self.num do
			local uf = self.frame[i]
			if uf.guiCreated then
				BUF_Factory:Hide(uf)
			end
		end
		
		self.BaseFrame:EnableMouse(true)
		self.BaseFrame:SetMovable(true)
		self.BaseFrame:SetResizable(true)
		
		self.Movable = true
	else
		self:DisableMove()
	end
end

function BUF_Area:DisableMove()
	self.BaseFrame:SetBackdrop(nil)
	self.Grip:Hide()
	self.Label:Hide()

	self.BaseFrame:EnableMouse(false)
	self.BaseFrame:SetMovable(false)
	self.BaseFrame:SetResizable(false)
	
	self.Movable = false
	self:Rearrange()
end

function BUF_Area:UpdateVisibility()
	if self.Movable then return end

	local InCombat = InCombatLockdown()
	
	if self.Enabled and self.Active then
		if not InCombat then self.BaseFrame:Show() end
		for i = 1, self.num do
			local uf = self.frame[i]
			if UnitExists(uf.unitId) then
				AuraCache:UpdateUnitAura(uf.unitId)			
				BUF_Factory:UpdateUnitFrameComplete(uf)
				BUF_Factory:CleanupUnitAura(uf, true)
			end			
		end
	else
		if not InCombat then self.BaseFrame:Hide() end
	end
end

function BUF_Area:Rearrange(initialization)
	if InCombatLockdown() and not initialization then
		self:UpdateVisibility()
	else
		-- if InCombatLockdown() then print("Rearrange in Combat"); end
		self:UpdatePlacement(initialization)
	end	
end

function BUF_Area:UpdatePlacement(initialization, order, maxsubgroup)
	-- ggf. Größe anpassen
	local displayRows
	if self.Horizontal then
		displayRows = math.max(1, self.MinDisplayRows, maxsubgroup or 0)
	else
		displayRows = 5
	end
	self.ufWidth = (self.Width - (self.MaxColumns - 1) * self.Spacer) / self.MaxColumns 
	self.ufHeight = (self.Height - (displayRows - 1) * self.Spacer) / displayRows 	

	for i = 1, self.num do
		BUF_Factory:UpdateUnitFrameSize(self.frame[i], self.ufWidth, self.ufHeight)
	end	

	-- Frames verschieben	
	local uf, uftarget, tookSpace
	local col = 1
	local row = 1
	local count = self.num
	if self.Targets then count = count / 2 end

	for index = 1, count do		
		if order then
			uf = self.frame[order[index]]
		else
			uf = self.frame[index]
		end
		BUF_Factory:UpdateDisplayUnitId(uf)
		local unitId = uf.unitId

		uf.Width = self.ufWidth
		uf.Height = self.ufHeight

		tookSpace = false						
		if UnitExists(unitId) and self.Active and self.Enabled then
			BUF_Factory:SetPoint(uf, self.BaseFrame, "TOPLEFT",
				(col - 1) * (self.ufWidth + self.Spacer),
				(row - 1) * (self.ufHeight + self.Spacer) * -1
			)
			BUF_Factory:Show(uf)
			tookSpace = true
		else
			if initialization and not self.LateCreation and self.Enabled then
				BUF_Factory:CreateUnitFrameGUI(uf, self.BaseFrame, "TOPLEFT",		
					(col - 1) * (self.ufWidth + self.Spacer),
					(row - 1) * (self.ufHeight + self.Spacer) * -1
				)	
				tookSpace = true
			end
			if self.LateCreation then
				-- BUF_Factory:Hide(uf)
			else
				if self.Enabled and self.Active then
					BUF_Factory:Show(uf)
					-- BUF_Factory:SecureHide(uf)			
				else
					BUF_Factory:Hide(uf)
				end
			end
		end
						
		if tookSpace or order then
			if self.Horizontal then
				col = col + 1
				if col > self.MaxColumns then
					col = 1
					row = row + 1
				end
			else
				row = row + 1
				if row > displayRows then
					row = 1
					col = col + 1
				end
			end

			if self.Targets then
				uftarget = self.frame[index + count]

				uftarget.Width = self.ufWidth
				uftarget.Height = self.ufHeight

				BUF_Factory:SetPoint(uftarget, self.BaseFrame, "TOPLEFT",
					(col - 1) * (self.ufWidth + self.Spacer),
					(row - 1) * (self.ufHeight + self.Spacer) * -1
				)			
				BUF_Factory:UpdateUnitFrameSize(uftarget, self.ufWidth, self.ufHeight)
				BUF_Factory:Show(uftarget)
				
				col = col + 1
				if col > self.MaxColumns then
					col = 1
					row = row + 1
				end				
			end
		end
	end
end

function BUF_Area:UpdateAll(func)
	if self.Enabled and self.Active then		
		for i = 1, self.num do
			local uf = self.frame[i]
			if uf.guiCreated then
				func(BUF_Factory, uf)
			end
		end
	end
end

function BUF_Area:UpdateUF(unitId, func)
	if self.Enabled and self.Active then
		local index = self.unitId[unitId] or 0		
		local uf = self.frame[index] 
		if uf and uf.unitId == unitId and uf.guiCreated then
			func(BUF_Factory, uf)	
		end

		index = self.vehicleId[unitId] or 0
		uf = self.frame[index] 
		if uf and uf.vehicleId == unitId and uf.guiCreated then
			func(BUF_Factory, uf)		
		end
	end
end

function BUF_Area:UpdateUnitFrame(unitId, force)
	if self.Enabled and self.Active then
		local index = self.unitId[unitId] or 0		
		local uf = self.frame[index] 
		if uf and uf.unitId == unitId then
			BUF_Factory:UpdateUnitFrame(uf)
			BUF_Factory:CleanupUnitAura(uf, force)
		end
	end
end

function BUF_Area:UpdateUnitFrameComplete(unitId)
	if self.Enabled and self.Active then
		local index = self.unitId[unitId] or 0		
		local uf = self.frame[index] 
		if uf and uf.unitId == unitId then
			BUF_Factory:UpdateUnitFrameComplete(uf)
		end
	end
end

function BUF_Area:UpdateAllUnitFrames(cleanupAura)
	if self.Enabled and self.Active then
		for i = 1, self.num do
			local uf = self.frame[i]
			BUF_Factory:UpdateDisplayUnitId(uf)
			BUF_Factory:UpdateUnitFrame(uf)
			if cleanupAura then
				BUF_Factory:CleanupUnitAura(uf)
			else
				BUF_Factory:UpdateCowndown(uf)
			end
		end
	end
end

function BUF_Area:ShowSpellsymbol(unitId, icon)
	if self.Enabled and self.Active then
		local index = self.unitId[unitId] or 0		
		local uf = self.frame[index] 
		if uf and uf.unitId == unitId then
			BUF_Factory:ShowSpellsymbol(uf, icon)
		end
	end
end

function BUF_Area:SetActive(isActivated)
	local InCombat = InCombatLockdown()

	if not (InCombat or self.Movable or not self.Enabled) then 
		if isActivated then	
			self.BaseFrame:Show()
		else
			self.BaseFrame:Hide()
		end
	end

	self.Active = isActivated
end