﻿-- Aura-Frame

BAF_Factory = {
	AuraSize = 14,

	CountFont = "Monofonto",
	CountFontSize = 10,
	CountFontFlags = "OUTLINE",
	CountFontR = 1,
	CountFontG = 1,
	CountFontB = 1,
	CountFontA = 1,

	CowndownFont = "Monofonto",
	CowndownFontSize = 10,
	CowndownFontFlags = "OUTLINE",
	CowndownFontR = 1,
	CowndownFontG = 1,
	CowndownFontB = 1,
	CowndownFontA = 1,
	
	-- Auren, bei denen der Cowndown die verstrichene Zeit hochzählt
	ForwardCowndown = {
		[70337] = true, [73912] = true, [73913] = true, [73914] = true, -- Necrotic Plague
	},
}

function BAF_Factory:ApplyOptions(options)
	self.CountFont = options.CountFont or self.CountFont
	self.CountFontSize = options.CountFontSize or self.CountFontSize
	self.CountFontFlags = options.CountFontFlags or self.CountFontFlags
	self.CountFontR = options.CountFontR or self.CountFontR
	self.CountFontG = options.CountFontG or self.CountFontG
	self.CountFontB = options.CountFontB or self.CountFontB
	self.CountFontA = options.CountFontA or self.CountFontA

	self.CowndownFont = options.CowndownFont or self.CowndownFont
	self.CowndownFontSize = options.CowndownFontSize or self.CowndownFontSize
	self.CowndownFontFlags = options.CowndownFontFlags or self.CowndownFontFlags
	self.CowndownFontR = options.CowndownFontR or self.CowndownFontR
	self.CowndownFontG = options.CowndownFontG or self.CowndownFontG
	self.CowndownFontB = options.CowndownFontB or self.CowndownFontB
	self.CowndownFontA = options.CowndownFontA or self.CowndownFontA
end

function BAF_Factory:CreateAuraFrame(parent, auraType)
	local aura = {		
		spellId = 0,
		caster = AuraCache.NILGUID,
		count = 0,
		duration = 0,
		expirationTime = 0,		
	}
	
	aura.auraType = auraType
	
	aura.BaseFrame = CreateFrame("Frame", nil, parent)
	aura.BaseFrame:SetWidth(self.AuraSize)
	aura.BaseFrame:SetHeight(self.AuraSize)
	
	-- Texture für Aura-Icon
	aura.Icon = aura.BaseFrame:CreateTexture()
	aura.Icon:SetAllPoints(aura.BaseFrame)

	-- Anzahl Aufladungen
	aura.Count = aura.BaseFrame:CreateFontString(nil, nil)
	aura.Count:SetPoint("BOTTOMRIGHT", aura.BaseFrame)
    aura.Count:SetFont(bUF.LSM:Fetch(bUF.LSM.MediaType.FONT, self.CountFont), self.CountFontSize, self.CountFontFlags)
	aura.Count:SetTextColor(self.CountFontR, self.CountFontG, self.CountFontB, self.CountFontA)

		-- Cooldown
    aura.Cooldown = CreateFrame("Cooldown", nil, aura.BaseFrame, "CooldownFrameTemplate")  
    aura.Cooldown:SetAllPoints(aura.BaseFrame)
    aura.Cooldown.noCooldownCount = true -- OmniCC weigert sich vermutlich eh, auf so kleine Cooldowns Zahlen zu schreiben
    
    -- Cowndown
    aura.Cowndown = aura.BaseFrame:CreateFontString(nil, "OVERLAY")
    aura.Cowndown:SetPoint("TOPLEFT", aura.BaseFrame)
    aura.Cowndown:SetFont(bUF.LSM:Fetch(bUF.LSM.MediaType.FONT, self.CowndownFont), self.CowndownFontSize, self.CowndownFontFlags)
	aura.Cowndown:SetTextColor(self.CowndownFontR, self.CowndownFontG, self.CowndownFontB, self.CowndownFontA)
    
    return aura
end

function BAF_Factory:UpdateStyle(aura)
    aura.Count:SetFont(bUF.LSM:Fetch(bUF.LSM.MediaType.FONT, self.CountFont), self.CountFontSize, self.CountFontFlags)
	aura.Count:SetTextColor(self.CountFontR, self.CountFontG, self.CountFontB, self.CountFontA)
	aura.Cowndown:SetFont(bUF.LSM:Fetch(bUF.LSM.MediaType.FONT, self.CowndownFont), self.CowndownFontSize, self.CowndownFontFlags)
	aura.Cowndown:SetTextColor(self.CowndownFontR, self.CowndownFontG, self.CowndownFontB, self.CowndownFontA)
end

function BAF_Factory:UpdateHeight(aura, newHeight)
	aura.BaseFrame:SetWidth(newHeight)
	aura.BaseFrame:SetHeight(newHeight)
end

function BAF_Factory:FillAuraFrame(aura)
	if aura.spellId == 0 then
		self:HideAuraFrame(aura)
		return
	end
	
	local _, icon = CSL_Factory:GetSpellName(aura.spellId)	

	aura.Icon:SetTexture(icon)
	if (aura.count or 0) > 1 then
		aura.Count:SetText(aura.count)
	else
		aura.Count:SetText("")
	end
	if aura.expirationTime then
		CooldownFrame_SetTimer(aura.Cooldown, aura.expirationTime - aura.duration, aura.duration, 1)
		aura.Cooldown:SetReverse(false)		
	end
	aura.Cooldown:SetAlpha(0.3)
	if (aura.expirationTime or 0) > 0 and (aura.duration or 0) > 0 then
		aura.Cooldown:Show()
	else
		aura.Cooldown:Hide()
	end
		
	aura.BaseFrame:Show()
	aura.Icon:Show()
	
	self:UpdateCowndown(aura)
end

function BAF_Factory:UpdateCowndown(aura)
	if (aura.expirationTime or 0) > 0 and (aura.duration or 0) > 0 then
		local seconds, critical
		local now = GetTime()

		if self.ForwardCowndown[aura.spellId] then
			-- bisherige Dauer		
			seconds = math.max(0, math.floor(now - aura.expirationTime + aura.duration))
			critical = seconds > 4
		else
			-- Verbleibende Zeit in Sekunden
			seconds = math.floor(aura.expirationTime - now)			
			critical = seconds < 5
		end
				
--		if critical then
--			aura.Cowndown:SetTextColor(1, 0, 0, 1)
--		else
			aura.Cowndown:SetTextColor(1, 1, 1, 1)
--		end	

		seconds = math.max(0, seconds)
		if seconds < 10 then
			aura.Cowndown:SetText(seconds)
		else
			aura.Cowndown:SetText("*")
		end
--		if seconds < 60 then
--			aura.Cowndown:SetText(seconds)
--		else
--			aura.Cowndown:SetFormattedText("%dm", math.floor(seconds / 60 + 0.5))
--		end
		return seconds >= 0
	else
		aura.Cowndown:SetText("")
		return true
	end
end

function BAF_Factory:UpdateAuraFrame(aura, spellId, caster, count, duration, expirationTime)
	aura.spellId = spellId
	aura.caster = caster
	aura.count = count
	aura.duration = duration
	aura.expirationTime = expirationTime
	
	self:FillAuraFrame(aura)
end

function BAF_Factory:CopyAuraFrame(aura, sourceAura)	
	aura.spellId = sourceAura.spellId
	aura.caster = sourceAura.caster
	aura.count = sourceAura.count
	aura.duration = sourceAura.duration
	aura.expirationTime = sourceAura.expirationTime
	
	self:FillAuraFrame(aura)
end

function BAF_Factory:HideAuraFrame(aura)
	aura.spellId = 0
	aura.caster = AuraCache.NILGUID
	aura.count = 0
	aura.duration = 0
	aura.expirationTime = 0
	
	aura.BaseFrame:Hide()
end

function BAF_Factory:AuraEquals(aura, spellId, caster)
	if caster then
		return aura.spellId == spellId and aura.caster == caster
	else
		return aura.spellId == spellId
	end
end