﻿-- Raidframes Layout Manager

local pairs, ipairs = pairs, ipairs

BUF_Roster = {
	num = 0, -- Anzahl Areas
	areas = {},	
	oRA_mt = {},
	classCounts = {},
	raidMembers = {},
	resurrection = {}, -- Gerezzter (UnitID) => Rezzer
	resSpellnames = { [(GetSpellInfo(50769))] = true, [(GetSpellInfo(20484))] = true, [(GetSpellInfo(7328))] = true, [(GetSpellInfo(2006))] = true, [(GetSpellInfo(2008))] = true, },
	
	partyIds = { "player", "party1", "party2", "party3", "party4", },	
	partyPetIds = { "pet", "partypet1", "partypet2", "partypet3", "partypet4", },
	partyDropDown = { PlayerFrameDropDown, PartyMemberFrame1DropDown, PartyMemberFrame2DropDown, PartyMemberFrame3DropDown, PartyMemberFrame4DropDown },
}

function BUF_Roster:RegisterArea(area)
	self.num = self.num + 1
	self.areas[self.num] = area
end

function BUF_Roster:UpdateByUnitID(unitId, funcName)
	local func = BUF_Factory[funcName]

	local guid = AuraCache:GetUnitGUID(unitId)
	local uil = AuraCache:GetUnitID(guid)
	if not uil then
		uil = { unitId }
	end

	for ai = 1, self.num do
		local ca = self.areas[ai]
		for k, _ in pairs(uil) do
			ca:UpdateUF(k, func)
		end
	end
end

function BUF_Roster:UpdateAll(funcName)
	local func = BUF_Factory[funcName]
	for ai = 1, self.num do
		local ca = self.areas[ai]
		ca:UpdateAll(func)
	end
end

function BUF_Roster:UpdateAuras(event, guid, auraType, debuffType, spellId, caster, count, duration, expirationTime)
	local uil = AuraCache:GetUnitID(guid)

	if uil then
		for ai = 1, self.num do
			local ca = self.areas[ai]
			
			for k, _ in pairs(uil) do
				local index = ca.unitId[k]
				local uf = ca.frame[index]

				if uf and AuraCache:GetUnitGUID(k) == guid then
					BUF_Factory:UpdateUnitAura(uf, event, auraType, debuffType, spellId, caster, count, duration, expirationTime)
				end
			end
		end
	end
end

function BUF_Roster:FastUpdateLife(guid, amount, spellId, incomingHeal)
	local uil = AuraCache:GetUnitID(guid)

	if uil then
		for ai = 1, self.num do
			local ca = self.areas[ai]
			
			for k, _ in pairs(uil) do
				local index = ca.unitId[k]
				local uf = ca.frame[index]

				if uf then
					BUF_Factory:FastUpdateLife(uf, amount, spellId, incomingHeal)
				end
			end
		end
	end
end

function BUF_Roster:ShowSpellsymbol(unitId, icon)
	for ai = 1, self.num do
		local ca = self.areas[ai]
		ca:ShowSpellsymbol(unitId, icon)
	end
end
function BUF_Roster:EnableMove()
	for ai = 1, self.num do
		local ca = self.areas[ai]
		ca:EnableMove()
	end
end

function BUF_Roster:DisableMove()
	for ai = 1, self.num do
		local ca = self.areas[ai]
		ca:DisableMove()
	end
end

function BUF_Roster:DragStop(areaFrame)
	areaFrame:StopMovingOrSizing()
	
	local point, relativeTo, relativePoint, xOfs, yOfs = areaFrame:GetPoint()
	local area = areaFrame.area	
	
	if not self.db.global.area then
		self.db.global.area = {}
	end
	if not self.db.global.area[area.Name] then
		self.db.global.area[area.Name] = {}
	end
	self.db.global.area[area.Name].point = point
	self.db.global.area[area.Name].relativePoint = relativePoint
	self.db.global.area[area.Name].x = xOfs
	self.db.global.area[area.Name].y = yOfs	
end

function BUF_Roster:GripMouseUp(area)
	area.BaseFrame:StopMovingOrSizing()
	
	if not self.db.global.area then
		self.db.global.area = {}
	end
	if not self.db.global.area[area.Name] then
		self.db.global.area[area.Name] = {}
	end
	
	area.Width = area.BaseFrame:GetWidth()
	area.Height = area.BaseFrame:GetHeight()
	self.db.global.area[area.Name].Width = area.Width
	self.db.global.area[area.Name].Height = area.Height
end

function BUF_Roster:UpdateClassCounts()	
	local unitId, className
	for _, v in ipairs(CLASS_SORT_ORDER) do
		self.classCounts[v] = 0
	end
	wipe(self.raidMembers)
	
	if GetNumRaidMembers() == 0 then
		for _, v in ipairs(self.partyIds) do
			unitId = v

			if UnitExists(unitId) then
				_, className = UnitClass(unitId)
				if className then
					self.classCounts[className] = self.classCounts[className] + 1
					self.raidMembers[(UnitName(unitId))] = unitId
				end
			end			
		end
	else
		for raidIndex = 1, MAX_RAID_MEMBERS do
			unitId = "raid" .. raidIndex

			if UnitExists(unitId) then
				_, className = UnitClass(unitId)
				if className then
					self.classCounts[className] = self.classCounts[className] + 1
					self.raidMembers[(UnitName(unitId))] = unitId
				end
			end
		end
	end
end

function BUF_Roster:RearrangeAll()
	for i = 1, self.num do
		local area = self.areas[i]
		area:ApplyOptions(self.db.global.area)
		area:Rearrange()
	end
end

function BUF_Roster:Rearrange()
	for i = 1, self.num do
		local area = self.areas[i]
		area:Rearrange()
	end
end

function BUF_Roster:UpdateUnitFrameStyle()
	for ai = 1, self.num do
		local ca = self.areas[ai]
		for i = 1, ca.num do
			local uf = ca.frame[i]
			BUF_Factory:UpdateUnitFrameStyle(uf)
			BUF_Factory:UpdateUnitFrameSize(uf, uf.Width, uf.Height)
		end
	end
end

function BUF_Roster:UpdateUnitFrame(unitId, force)
	for ai = 1, self.num do
		local ca = self.areas[ai]
		ca:UpdateUnitFrame(unitId, force)
	end
end

function BUF_Roster:UpdateUnitFrameComplete(unitId)
	for ai = 1, self.num do
		local ca = self.areas[ai]
		ca:UpdateUnitFrameComplete(unitId)
	end
end

function BUF_Roster:UpdateAllUnitFrames(cleanupAura)
	for ai = 1, self.num do
		local ca = self.areas[ai]
		ca:UpdateAllUnitFrames(cleanupAura)
	end
end

function BUF_Roster:UpdateAllSecureButton()
	for _, ca in pairs(self.areas) do
		for i = 1, ca.num do
			BUF_Factory:UpdateSecureButton(ca.frame[i])
		end
	end
end
