﻿bUF = LibStub("AceAddon-3.0"):NewAddon("BakorsUF", "AceComm-3.0", "AceConsole-3.0", "AceEvent-3.0")

local pairs, ipairs = pairs, ipairs
local sfind, ssub = string.find, string.sub

function bUF:OnInitialize()
	local acedb = LibStub("AceDB-3.0")
	local acecfg = LibStub("AceConfig-3.0")
	local acecd = LibStub("AceConfigDialog-3.0")
	self.LSM = LibStub("LibSharedMedia-3.0")

	self.db = acedb:New("bufDB", defaults, "Default")
	BUF_Roster.db = self.db
	BUF_Factory.db = self.db
	BUF_Factory:CreateTemplates()

	self:SanatizeDB()
	
	BUF_Factory:Init()
	
	-- Register custom media
	self.LSM:Register(self.LSM.MediaType.FONT, "Myriad Condensed Web", "Interface\\AddOns\\BakorsUF\\fonts\\barframes.ttf")
	self.LSM:Register(self.LSM.MediaType.FONT, "Monofonto", "Interface\\AddOns\\BakorsUF\\fonts\\MONOFONT.TTF")
	self.LSM:Register(self.LSM.MediaType.STATUSBAR, "Smooth", "Interface\\AddOns\\BakorsUF\\images\\Smooth.tga")
	self.LSM:Register(self.LSM.MediaType.STATUSBAR, "Raid Bar Hp Fill", "Interface\\RaidFrame\\Raid-Bar-Hp-Fill")
	self.LSM:Register(self.LSM.MediaType.STATUSBAR, "Raid Bar Resource Fill", "Interface\\RaidFrame\\Raid-Bar-Resource-Fill")
	
	self.Movable = false
	self:BaseOptions()
	
	self:RegisterChatCommand("buf", "OnSlash")	
	
	self.checkMode = false
	self.areas = {}

	self.player = BUF_AreaPlayer:new()
	self.player.Width = 200
	self.player.Height = 40
	self.areas[self.player.Name] = self.player
	BUF_Roster:RegisterArea(self.player)

	self.target = BUF_AreaTarget:new()
	self.target.Width = 200
	self.target.Height = 40
	self.areas[self.target.Name] = self.target
	BUF_Roster:RegisterArea(self.target)

	self.targettarget = BUF_AreaTargetTarget:new()
	self.targettarget.Width = 200
	self.targettarget.Height = 40
	self.areas[self.targettarget.Name] = self.targettarget
	BUF_Roster:RegisterArea(self.targettarget)

	self.focus = BUF_AreaFocus:new()
	self.focus.Width = 200
	self.focus.Height = 40
	self.areas[self.focus.Name] = self.focus
	BUF_Roster:RegisterArea(self.focus)
	
	self.raid = BUF_AreaRaid:new()	
	self.raid.Width = 400
	self.raid.Height = 125
	self.areas[self.raid.Name] = self.raid
	BUF_Roster:RegisterArea(self.raid)
	
	self.raidpet = BUF_AreaRaidPet:new()
	self.raidpet.Width = 80
	self.raidpet.Height = 125
	self.areas[self.raidpet.Name] = self.raidpet
	BUF_Roster:RegisterArea(self.raidpet)
	
	self.party = BUF_AreaParty:new()
	self.party.Width = 400
	self.party.Height = 125
	self.areas[self.party.Name] = self.party
	BUF_Roster:RegisterArea(self.party)
	
	self.partypet = BUF_AreaPartyPet:new()
	self.partypet.Width = 80
	self.partypet.Height = 125
	self.areas[self.partypet.Name] = self.partypet	
	BUF_Roster:RegisterArea(self.partypet)
	
	self.mt = BUF_AreaMT:new()
	self.mt.Width = 80
	self.mt.Height = 125
	self.areas[self.mt.Name] = self.mt
	BUF_Roster:RegisterArea(self.mt)

	self.boss = BUF_AreaBoss:new()
	self.boss.Width = 80
	self.boss.Height = 125
	self.areas[self.boss.Name] = self.boss
	BUF_Roster:RegisterArea(self.boss)

	self:SanatizeDB2()
	
	BAF_Factory:ApplyOptions(self.db.global.aura)
	for _, v in pairs(self.areas) do
		v:CreateGUI(self.db.global.area)
		BUF_Roster:InsertOptions(self.options, v)
	end

	self:UpdateCans()
	
	acecfg:RegisterOptionsTable("bUF", self.options)
	acecd:AddToBlizOptions("bUF")	
end

function bUF:SanatizeDB()
	if not self.db.global.clickcast then
		self.db.global.clickcast = {}
	end
	for _, v in ipairs(CLASS_SORT_ORDER) do
		if not self.db.global.clickcast[v] then
			self.db.global.clickcast[v] = {}
		end
	end		
	if not self.db.global.buffs then
		self.db.global.buffs = {}
	end
	if not self.db.global.aura then
		self.db.global.aura = {}
	end
	for _, v in ipairs(CLASS_SORT_ORDER) do
		if not self.db.global.buffs[v] then
			self.db.global.buffs[v] = {}
		end
	end			
	if not self.db.global.templates then
		self.db.global.templates = {}
	end	
	for k, _ in pairs(BUF_Factory.templates) do
		if not self.db.global.templates[k] then
			self.db.global.templates[k] = {}
		end
	end	
	if not self.db.global.area then
		self.db.global.area = {}
	end
	if not self.db.global.debuffblacklist then
		self.db.global.debuffblacklist = {}				
	end	
	if not self.db.global.buffwhitelist then
		self.db.global.buffwhitelist = {}	
	end
	if not self.db.global.emergencydebuff then
		self.db.global.emergencydebuff = {}
	end
	if not self.db.global.spellwhitelist then
		self.db.global.spellwhitelist = {}
	end
end

function bUF:SanatizeDB2()
	for k, _ in pairs(self.areas) do
		if not self.db.global.area[k] then
			self.db.global.area[k] = {}
		end
	end	
end

function bUF:OnEnable()
	-- Events bzgl. Raid
	self:RegisterEvent("PARTY_LEADER_CHANGED")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	self:RegisterEvent("PARTY_CONVERTED_TO_RAID")
	self:RegisterEvent("RAID_ROSTER_UPDATE")
	self:RegisterEvent("RAID_TARGET_UPDATE")
	self:RegisterEvent("UNIT_PET")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_UPDATE_RESTING")
	self:RegisterEvent("READY_CHECK")

	self:RegisterEvent("PLAYER_FLAGS_CHANGED")	
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
	self:RegisterEvent("UNIT_FACTION")
	self:RegisterEvent("UNIT_TARGET")
	self:RegisterEvent("UNIT_HEALTH")
	self:RegisterEvent("UNIT_MAXHEALTH")
	self:RegisterEvent("UNIT_POWER")
	self:RegisterEvent("UNIT_MAXPOWER")
	self:RegisterEvent("UNIT_NAME_UPDATE")
	self:RegisterEvent("UNIT_LEVEL")
	self:RegisterEvent("UNIT_CLASSIFICATION_CHANGED")
	self:RegisterEvent("UNIT_DISPLAYPOWER")
	self:RegisterEvent("UNIT_THREAT_SITUATION_UPDATE")
	self:RegisterEvent("UNIT_ENTERED_VEHICLE")
	self:RegisterEvent("UNIT_EXITED_VEHICLE")
	self:RegisterEvent("UNIT_HEAL_PREDICTION")
	
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
	self:RegisterEvent("INSTANCE_ENCOUNTER_ENGAGE_UNIT")

	self:RegisterEvent("UNIT_SPELLCAST_SENT")
	self:RegisterEvent("UNIT_SPELLCAST_FAILED")
	self:RegisterEvent("UI_ERROR_MESSAGE")

	-- oRA2 Kommunikation abhören	
	self:RegisterComm("oRA")
	self:RegisterComm("CTRA")
	
	self:ApplyIncomingHeal()
		
	AuraCache:RegisterListener("ANY", 
		function (event, timestamp, GUID, auraType, debuffType, spellId, caster, count, duration, expirationTime)
			self:AuraCacheNotifier(event, timestamp, GUID, auraType, debuffType, spellId, caster, count, duration, expirationTime)
		end
	)		
	AuraCache:RegisterListenerScan(function (unitId) self:AuraCacheScan(unitId) end)

	for k, v in pairs(BUF_Factory.templates) do
		BUF_Factory:ApplyOptions(k)
	end

	self:PARTY_MEMBERS_CHANGED()
end 

function bUF:OnDisable()
	self:UnregisterEvent("PARTY_LEADER_CHANGED")
	self:UnregisterEvent("PARTY_MEMBERS_CHANGED")
	self:UnregisterEvent("PARTY_CONVERTED_TO_RAID")
	self:UnregisterEvent("RAID_ROSTER_UPDATE")
	self:UnregisterEvent("RAID_TARGET_UPDATE")
	self:UnregisterEvent("UNIT_PET")
	self:UnregisterEvent("PLAYER_REGEN_ENABLED")
	self:UnregisterEvent("PLAYER_REGEN_DISABLED")
	self:UnregisterEvent("PLAYER_UPDATE_RESTING")
	self:UnregisterEvent("READY_CHECK")

	self:UnregisterEvent("PLAYER_FLAGS_CHANGED")
	self:UnregisterEvent("PLAYER_TARGET_CHANGED")
	self:UnregisterEvent("UNIT_FACTION")
	self:UnregisterEvent("UNIT_TARGET")
	self:UnregisterEvent("UNIT_HEALTH")
	self:UnregisterEvent("UNIT_MAXHEALTH")
	self:UnregisterEvent("UNIT_POWER")
	self:UnregisterEvent("UNIT_MAXPOWER")
	self:UnregisterEvent("UNIT_NAME_UPDATE")
	self:UnregisterEvent("UNIT_LEVEL")
	self:UnregisterEvent("UNIT_CLASSIFICATION_CHANGED")
	self:UnregisterEvent("UNIT_DISPLAYPOWER")
	self:UnregisterEvent("UNIT_THREAT_SITUATION_UPDATE")
	self:UnregisterEvent("UNIT_ENTERED_VEHICLE")
	self:UnregisterEvent("UNIT_EXITED_VEHICLE")
	self:UnregisterEvent("UNIT_HEAL_PREDICTION")
	
	self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")	
	self:UnregisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
	self:UnregisterEvent("INSTANCE_ENCOUNTER_ENGAGE_UNIT")

	self:UnregisterEvent("UNIT_SPELLCAST_SENT")
	self:UnregisterEvent("UNIT_SPELLCAST_FAILED")
	self:UnregisterEvent("UI_ERROR_MESSAGE")
end 

function bUF:UpdateCans()
	local _, playerClass = UnitClass("player")

	self.canCurse = false
	self.canMagic = false
	self.canDisease = false
	self.canPoison = false

	if playerClass == "DRUID" then
		self.canCurse = true
		self.canPoison = true

		local _, _, _, _, currentRank = GetTalentInfo(3, 17) -- Heilung der Natur
		self.canMagic = currentRank ~= 0
	elseif playerClass == "PALADIN" then
		self.canDisease = true
		self.canPoison = true

		local _, _, _, _, currentRank = GetTalentInfo(1, 14) -- Sacred Cleansing
		self.canMagic = currentRank ~= 0
	elseif playerClass == "PRIEST" then
		self.canMagic = true
		self.canDisease = true
	elseif playerClass == "SHAMAN" then
		self.canCurse = true

		local _, _, _, _, currentRank = GetTalentInfo(3, 12)
		self.canMagic = currentRank ~= 0
	elseif playerClass == "MAGE" then
		self.canCurse = true
	end
end

function bUF:ApplyMovable()
	if self.Movable then
		BUF_Roster:EnableMove()
	else
		BUF_Roster:DisableMove()
		self:PARTY_MEMBERS_CHANGED()
	end
end

function bUF:ApplyIncomingHeal()
	-- scheint überflüssig zu sein nun
end

function bUF:UNIT_HEAL_PREDICTION(msg, arg1)	
	if self.db.global.IncomingHeal then
		BUF_Roster:UpdateByUnitID(arg1, "UpdateIncomingHeal")		
	end
end

function bUF:GetIncomingHeal(unitId)
	local heal
	local myheal = (UnitGetIncomingHeals(unitId, "player") or 0) 
	if self.db.global.IncomingHealOnlyOthers then
		heal = (UnitGetIncomingHeals(unitId) or 0) - myheal
	else
		heal = myheal
	end
	return heal
end

function bUF:SetCheckMode(checkMode)
	if checkMode and InCombatLockdown() then checkMode = false end 

	self.checkMode = checkMode
	BUF_Roster:UpdateAllUnitFrames(true)
end

function bUF:OnSlash(args)		
	if args == "check" then
		if self.checkMode then
			self:SetCheckMode(false)
		else
			self:SetCheckMode(true)
		end
	elseif args == "test" then
		self:AuraTest()
	else
		LibStub("AceConfigDialog-3.0"):Open("bUF")
	end
end

function bUF:SplitMessage(message)
	local tokens = {}
	local spacePos, lastSpacePos, tokenIndex

	tokenIndex = 0
	lastSpacePos = 0
	spacePos = sfind(message, " ", lastSpacePos + 1, true)
	while spacePos or lastSpacePos do
		tokenIndex = tokenIndex + 1
		if spacePos then
			tokens[tokenIndex] = ssub(message, lastSpacePos + 1, spacePos - 1)
		else
			tokens[tokenIndex] = ssub(message, lastSpacePos + 1)
		end

		lastSpacePos = spacePos
		if lastSpacePos then
			spacePos = sfind(message, " ", lastSpacePos + 1, true)
		end
	end
	
	return tokens
end

function bUF:OnCommReceived(prefix, message, distribution, sender)
	if prefix == "CTRA" then
	
		-- Empfangene Nachricht splitten
		local tokens = self:SplitMessage(message)		
		if not tokens[1] then return end
		
		if tokens[1] == "SET" then
			BUF_Roster.oRA_mt[tonumber(tokens[2])] = tokens[3]
		elseif tokens[1] == "R" then
			if tonumber(tokens[2]) then
				BUF_Roster.oRA_mt[tonumber(tokens[2])] = nil
			else
				for k, v in pairs(BUF_Roster.oRA_mt) do
					if v == tokens[2] then
						BUF_Roster.oRA_mt[k] = nil
						break
					end
				end
			end
		elseif tokens[1] == "RES" and tokens[2] then
			local target = tokens[2]
			local dashPos = sfind(target, "-", 1, true)
			if dashPos then	target = ssub(target, 1, dashPos - 1) end
			local resTarget = BUF_Roster.raidMembers[target]
			if resTarget then
				BUF_Roster.resurrection[resTarget] = sender
			end
		elseif tokens[1] == "RESNO" then
			-- todo irgendwann mal
		end
		
		self.mt:UpdateMainTank()
		self.mt:Rearrange()
	end
end

function bUF:PARTY_LEADER_CHANGED()
	BUF_Roster:UpdateAll("UpdateIcons")
end

function bUF:PARTY_MEMBERS_CHANGED()
	local isParty = GetNumRaidMembers() == 0
	self.party:SetActive(isParty)
	self.raid:SetActive(not isParty)
	self.partypet:SetActive(isParty)
	self.raidpet:SetActive(not isParty)

	BUF_Roster:UpdateClassCounts()
	self.mt:UpdateMainTank()
	BUF_Roster:Rearrange()
end

function bUF:PARTY_CONVERTED_TO_RAID()
	self.raid:SetActive(true)
	self.party:SetActive(false)
	self.partypet:SetActive(false)
	self.raidpet:SetActive(true)
	
	BUF_Roster:UpdateClassCounts()
	self.mt:UpdateMainTank()
	BUF_Roster:Rearrange()
end

function bUF:RAID_ROSTER_UPDATE()
	BUF_Roster:UpdateClassCounts()
	self.mt:UpdateMainTank()
	BUF_Roster:Rearrange()
end

function bUF:INSTANCE_ENCOUNTER_ENGAGE_UNIT()
	self.boss:Rearrange()
end

function bUF:RAID_TARGET_UPDATE(msg, arg1)
	if arg1 then
		BUF_Roster:UpdateByUnitID(arg1, "UpdateIcons")		
	else
		BUF_Roster:UpdateAll("UpdateIcons")
	end
end

function bUF:UNIT_PET()
	BUF_Roster:Rearrange()
end

function bUF:PLAYER_REGEN_ENABLED()
	self.mt:UpdateMainTank()
	BUF_Roster:Rearrange()

	-- In Combat Icon aktualisieren
	BUF_Roster:UpdateByUnitID("player", "UpdateIcons")
end

function bUF:PLAYER_REGEN_DISABLED()
	self:SetCheckMode(false)

	-- In Combat Icon aktualisieren
	BUF_Roster:UpdateByUnitID("player", "UpdateIcons")
end

function bUF:PLAYER_UPDATE_RESTING()
	BUF_Roster:UpdateByUnitID("player", "UpdateIcons")
end

function bUF:READY_CHECK(msg, arg1, arg2)
	self:SetCheckMode(true)
end

function bUF:PLAYER_FLAGS_CHANGED(msg, arg1)
	BUF_Roster:UpdateByUnitID(arg1, "UpdateIcons")
end

function bUF:PLAYER_TARGET_CHANGED(msg, arg1)
end

function bUF:UNIT_FACTION(msg, arg1)
	BUF_Roster:UpdateByUnitID(arg1, "UpdateIcons")
end

function bUF:UNIT_TARGET(msg, arg1)
	BUF_Roster:UpdateAll("UpdateTarget")
end
	
----------------------------------------------------------------------
-- Events bzgl. Health Bar
----------------------------------------------------------------------
function bUF:UNIT_HEALTH(msg, arg1)
	BUF_Roster:UpdateByUnitID(arg1, "UpdateHealth")
end

function bUF:UNIT_MAXHEALTH(msg, arg1)
	BUF_Roster:UpdateByUnitID(arg1, "UpdateMaxHealth")
end

----------------------------------------------------------------------
-- Events bzgl. Energy Bar
----------------------------------------------------------------------
function bUF:UNIT_POWER(msg, arg1, arg2)
	BUF_Roster:UpdateByUnitID(arg1, "UpdatePower")
end

function bUF:UNIT_MAXPOWER(msg, arg1, arg2)
	BUF_Roster:UpdateByUnitID(arg1, "UpdateMaxPower")
end

function bUF:UNIT_DISPLAYPOWER(msg, arg1)
	BUF_Roster:UpdateByUnitID(arg1, "UpdatePowerDisplay")
end

----------------------------------------------------------------------
-- Events bzgl. Threat Bar
----------------------------------------------------------------------
function bUF:UNIT_THREAT_SITUATION_UPDATE(msg, arg1)
	if arg1 then
		BUF_Roster:UpdateByUnitID(arg1, "UpdateThreat")
	end
end

function bUF:UNIT_ENTERED_VEHICLE(msg, arg1, arg2, arg3)
	BUF_Roster:UpdateByUnitID(arg1, "UpdateDisplayUnitId")	
end

function bUF:UNIT_EXITED_VEHICLE(msg, arg1)
	BUF_Roster:UpdateByUnitID(arg1, "UpdateDisplayUnitId")	
end

function bUF:UNIT_NAME_UPDATE(msg, arg1)
	BUF_Roster:UpdateByUnitID(arg1, "UpdateName")	
end

function bUF:UNIT_LEVEL(msg, arg1)
	BUF_Roster:UpdateByUnitID(arg1, "UpdateStatus")	
end

function bUF:UNIT_CLASSIFICATION_CHANGED(msg, arg1)
	BUF_Roster:UpdateByUnitID(arg1, "UpdateStatus")	
end

function bUF:COMBAT_LOG_EVENT_UNFILTERED(event, timestamp, type, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, ...)
	-- Fast update for damage taken	
	local spellId, spellName, spellSchool
	local amount, overAmount
	
	if type == "SWING_DAMAGE" then
		amount = -(...)
	elseif (type == "RANGE_DAMAGE" or type == "SPELL_DAMAGE" or type == "SPELL_PERIODIC_DAMAGE" or type == "SPELL_BUILDING_DAMAGE") then
		spellId, spellName, spellSchool = select(1, ...)
		amount = -select(4, ...)
	elseif (type == "SPELL_HEAL" or type == "SPELL_PERIODIC_HEAL") then
		amount, overAmount = select(4, ...)
		amount = amount - overAmount
	elseif (type == "SPELL_CAST_START") then
		spellId, spellName, spellSchool = select(1, ...)
		if spellId == 57426 then -- Fish Feast
			self:SetCheckMode(true)

			local canspeak = IsRaidLeader() or IsRaidOfficer()
			if canspeak and GetNumRaidMembers() > 0 and self.db.global.AnnounceFeast then
				SendChatMessage((sourceName or "Somebody") .. " prepares a Fish Feast!", "RAID_WARNING")
			end
		end
	else
		return
	end
	
	if (amount or 0) ~= 0 then
		BUF_Roster:FastUpdateLife(destGUID, amount, spellId)
	end
end

function bUF:ACTIVE_TALENT_GROUP_CHANGED()
	self:UpdateCans()
end

local lastFriendlySpellcastSpell
local lastFriendlySpellcastTarget
local lastSpellError
function bUF:UNIT_SPELLCAST_SENT(msg, arg1, arg2, arg3, arg4)
	if arg1 == "player" then
		--ggf. Realmnamen entfernen samt "-"
		local dashPos = sfind(arg4, "-", 1, true)
		if dashPos then	arg4 = ssub(arg4, 1, dashPos - 1) end
		lastFriendlySpellcastTarget = BUF_Roster.raidMembers[arg4]
		if lastFriendlySpellcastTarget then
			lastFriendlySpellcastSpell = arg2
		else
			lastFriendlySpellcastSpell = nil
		end
	end
	-- Resurrection Monitor
	if BUF_Roster.resSpellnames[arg2] then
		local dashPos = sfind(arg4, "-", 1, true)
		if dashPos then	arg4 = ssub(arg4, 1, dashPos - 1) end
		local resTarget = BUF_Roster.raidMembers[arg4]
		if resTarget then
			BUF_Roster.resurrection[resTarget] = arg1
		end
	end
end

function bUF:UNIT_SPELLCAST_FAILED(msg, arg1, arg2, arg3)
	if arg1 == "player" and arg2 == lastFriendlySpellcastSpell then
		if lastSpellError == SPELL_FAILED_LINE_OF_SIGHT then
			BUF_Roster:ShowSpellsymbol(lastFriendlySpellcastTarget, "Interface\\Buttons\\UI-GroupLoot-Pass-Up")
		end
	end
end

function bUF:UI_ERROR_MESSAGE(msg, arg1)
	lastSpellError = arg1
end

function bUF:AuraCacheNotifier(event, timestamp, guid, auraType, debuffType, spellId, caster, count, duration, expirationTime)
	BUF_Roster:UpdateAuras(event, guid, auraType, debuffType, spellId, caster, count, duration, expirationTime)
end

function bUF:AuraCacheScan(unitId)
	BUF_Roster:UpdateUnitFrame(unitId, true)
end

function bUF:AuraTest()
	for i = 1, 40 do
		local id = "raid" .. i
		local guid = UnitGUID(id)
		if guid then
			for _, v in pairs(self.areas) do
				for _, uf in pairs(v.units) do
					-- BUF_Roster:UpdateAuras(v, "SPELL_AURA_APPLIED", guid, "DEBUFF", "Disease", 70337, AuraCache.NILGUID, 1, 0, 0)

					if uf.Debuffs then
						uf.debuffCount = 3
						for j = 1, 3 do
							BAF_Factory:UpdateAuraFrame(uf.Debuffs[j], 
									70337, AuraCache.NILGUID, 1, 0, 0
								)	
						end
					end
				end
			end
		end
	end
end