﻿-- Option dialog definitions

function bUF:CreateSpellList(root, childName, listName, list, default)
	root[childName] = {
		name = listName,
		desc = "Manage the " .. listName,
		type = "group",	
		args = {
			spellList = {
				order = 0,
				name = listName,
				type = "select",
				values = {},
				get = function(info)
					return root[childName].args.spellList.selected
				end,
				set = function(info, val)
					root[childName].args.spellList.selected = val
					local spellName, _, icon = GetSpellInfo(val)
					if not spellName then
						spellName = val
					end
					root[childName].args.Description.name = spellName .. "\n(" .. val .. ")"
					root[childName].args.Description.image = icon					
				end,
			},
			Description = {	
				order = 1,
				name = "",
				type = "description",
				image = "",
			},
			hdrEdit = {
				order = 2,
				name = "Edit list",
				type = "header",
			},
			inputName = {
				order = 3,
				name = "Spell Name",
				desc = "Enter Spell Name or SpellId",
				type = "input",   
				confirm = false,     
				set = function(info,val) root[childName].args.inputName.newVal = val end,
				get = function(info) return root[childName].args.inputName.newVal end,
			},
			cmdAdd = {
				order = 4,
				name = "Add",
				type = "execute",
				func = function()
					if root[childName].args.inputName.newVal then
						self:AddSpell(root[childName], list, root[childName].args.inputName.newVal)								
					end
				end,
			},												
			cmdRemove = {
				order = 5,
				name = "Remove",
				type = "execute",
				func = function(info)
					self:RemoveSpell(root[childName], list, root[childName].args.spellList.selected)
					root[childName].args.Description.name = ""
					root[childName].args.Description.image = ""
				end,
			},							
			cmdResetDefaults = {
				order = 6,
				name = "Reset to defaults",
				type = "execute",
				func = function()
					-- zuerst alle bisherigen Spells löschen
					for k, v in pairs(list) do
						self:RemoveSpell(root[childName], list, k)
					end
						
					-- dann Standardliste einfüllen
					for k, v in pairs(default) do
						self:AddSpell(root[childName], list, k)
					end
				end,
			},				
		},
	}

	for k, _ in pairs(list) do
		self:AddSpell(root[childName], list, k)
	end	
end

function bUF:PurgeSpellCache()
	BUF_Factory:PurgeSpellCache()
end

function bUF:AddSpell(anchor, list, spell)
	local spellkey = "" .. spell
	local spellName, _, icon = GetSpellInfo(spell)
	if not spellName then
		spellName = spell
	end
	if tonumber(spell) then spell = tonumber(spell) end
	list[spell] = true
	anchor.args.spellList.values[spellkey] = spellName

	self:PurgeSpellCache()
end

function bUF:RemoveSpell(anchor, list, spell)
	local spellkey = "" .. spell
	if tonumber(spell) then spell = tonumber(spell) end
	list[spell] = nil
	anchor.args.spellList.values[spellkey] = nil

	self:PurgeSpellCache()
end

function bUF:BaseOptions()
	self.options = {
		type = "group",
		args = {
			locked = {
				name = "Setup Mode",
				desc = "Size and position adjustments possible",
				type = "toggle",
				set = function(info,val) self.Movable = val; self:ApplyMovable() end,
				get = function(info) return self.Movable end
			},	
			templates = {
				name = "Templates",
				desc = "Adjust unit frame settings",
				type = "group",
				args = {
					inputName = {
						order = 3,
						name = "Template Name",
						desc = "Enter Template Name",
						type = "input",   
						confirm = false,     
						set = function(info,val) self.options.args.templates.args.inputName.newVal = val end,
						get = function(info) return self.options.args.templates.args.inputName.newVal end,
					},
					cmdAdd = {
						order = 4,
						name = "Add",
						type = "execute",
						func = function()
							local newTemplate = self.options.args.templates.args.inputName.newVal
							if newTemplate and not BUF_Factory.templates[newTemplate] then
								local tpl = BUF_Factory:CreateTemplate(newTemplate)
								bUF:TemplateOptions(newTemplate, tpl)
							end
						end,
					},												
					cmdRemove = {
						order = 5,
						name = "Remove",
						type = "execute",
						func = function(info)
							local delTemplate = self.options.args.templates.args.inputName.newVal
							if delTemplate and delTemplate ~= "standard" then
							end
						end,
					},		
				},
			},		
			areas = {
				name = "Areas",
				desc = "Configure the areas",
				type = "group",
				args = {},
			},
			clickcast = {
				name = "Clickcast",
				desc = "Configure the mouse actions",
				type = "group",
				args = {},
			},
			buffs = {
				name = "Buffs",
				desc = "Configure the buffs to display",
				type = "group",
				args = {},
			},	
		}   
	}
	
	self:CreateSpellList(self.options.args, "buffwhitelist", "Buff whitelist", self.db.global.buffwhitelist, BUF_Factory.BuffWhitelist)
	self:CreateSpellList(self.options.args, "debuffblacklist", "Debuff blacklist", self.db.global.debuffblacklist, BUF_Factory.DebuffBlacklist)
	self:CreateSpellList(self.options.args, "emergencydebuff", "Emergency Debuffs", self.db.global.emergencydebuff, BUF_Factory.EmergencyDebuff)
	self:CreateSpellList(self.options.args, "spellwhitelist", "Spell whitelist", self.db.global.spellwhitelist, BUF_Factory.SpellWhitelist)
	
	self:IncomingHealOptions()
	self:BuffCheckOptions()
	self:AuraOptions()
		
	for k, v in pairs(BUF_Factory.templates) do
		self:TemplateOptions(k, v)
	end
	
	for _, v in ipairs(CLASS_SORT_ORDER) do
		self:ClickcastOptions(v)
		self:BuffOptions(v)
	end
end

function bUF:TemplateOptions(templateName, template)
	self.options.args.templates.args[templateName] = {
		name = templateName,
		desc = "Unit frame settings for " .. template.Name,
		type = "group",
		args = {
			Detailed = {
				order = 0,
				name = "Detailed",
				desc = "Show detailed unit frames",
				type = "toggle",
				get = function(info)
					return self.db.global.templates[info[#info-1]].Detailed
				end,
				set = function(info,val)
					self.db.global.templates[info[#info-1]].Detailed = val
					BUF_Factory:ApplyOptions(info[#info-1])							
				end,									
			},
			BarTexture = {
				order = 1,
				type = "select",
				dialogControl = "LSM30_Statusbar",
				name = "Bar Texture",
				desc = "Choose the bar texture for health and energy bars",
				values = self.LSM:HashTable(self.LSM.MediaType.STATUSBAR),
				get = function(info)
					return self.db.global.templates[info[#info-1]].BarTexture
				end,
				set = function(info,val)
					 self.db.global.templates[info[#info-1]].BarTexture = val; BUF_Factory:ApplyOptions(info[#info-1])
				end,
			},
			BuffBarHeight = {
				order = 2,
				name = "Buff Bar Height",
				desc = "Choose Buff Bar Height in Percent of Total Height",
				type = "range",
				min = 0,
				max = 50,
				step = 1,
				bigStep = 1,
				set = function(info,val) self.db.global.templates[info[#info-1]].BuffBarHeight = val; BUF_Factory:ApplyOptions(info[#info-1]) end,
				get = function(info) return (self.db.global.templates[info[#info-1]].BuffBarHeight or 0) end
			},	
			EnergyBarHeight = {
				order = 3,
				name = "Energy Bar Height",
				desc = "Choose Energy Bar Height in Percent of Total Height",
				type = "range",
				min = 0,
				max = 50,
				step = 1,
				bigStep = 1,
				set = function(info,val) self.db.global.templates[info[#info-1]].EnergyBarHeight = val; BUF_Factory:ApplyOptions(info[#info-1]) end,
				get = function(info) return (self.db.global.templates[info[#info-1]].EnergyBarHeight or 0) end
			},	
			OnlyManaBar = {
				order = 4,
				name = "Only Mana",
				desc = "Show only mana bars",
				type = "toggle",
				get = function(info)
					return self.db.global.templates[info[#info-1]].OnlyManaBar
				end,
				set = function(info,val)
					self.db.global.templates[info[#info-1]].OnlyManaBar = val
					BUF_Factory:ApplyOptions(info[#info-1])							
				end,									
			},	
			ThreatBarHeight = {
				order = 5,
				name = "Threat Bar Height",
				desc = "Choose Threat Bar Height in Percent of Total Height",
				type = "range",
				min = 0,
				max = 50,
				step = 1,
				bigStep = 1,
				set = function(info,val) self.db.global.templates[info[#info-1]].ThreatBarHeight = val; BUF_Factory:ApplyOptions(info[#info-1]) end,
				get = function(info) return (self.db.global.templates[info[#info-1]].ThreatBarHeight or 0) end
			},	
			TargetThreat = {
				order = 6,
				name = "Target Threat",
				desc = "Show the exact threat to player's target (global threat otherwise)",
				type = "toggle",
				get = function(info)
					return self.db.global.templates[info[#info-1]].TargetThreat
				end,
				set = function(info,val)
					self.db.global.templates[info[#info-1]].TargetThreat = val
					BUF_Factory:ApplyOptions(info[#info-1])							
				end,									
			},										
			OnlyRemovable = {
				order = 7,
				name = "Only Removable",
				desc = "Debuff type colored bars only for debuffs removable by the player",
				type = "toggle",
				get = function(info)
					return self.db.global.templates[info[#info-1]].OnlyRemovable
				end,
				set = function(info,val)
					self.db.global.templates[info[#info-1]].OnlyRemovable = val
					BUF_Factory:ApplyOptions(info[#info-1])							
				end,									
			},			
			MaxDebuffs = {
				order = 8,
				name = "Max Debuffs",
				desc = "Choose Maximum Debuffs to show",
				type = "range",
				min = 0,
				max = 10,
				step = 1,
				bigStep = 1,
				set = function(info,val) self.db.global.templates[info[#info-1]].MaxDebuffs = val; BUF_Factory:ApplyOptions(info[#info-1]) end,
				get = function(info) return (self.db.global.templates[info[#info-1]].MaxDebuffs or 0) end
			},				
			MaxBuffs = {
				order = 9,
				name = "Max Buffs",
				desc = "Choose Maximum Buffs to show",
				type = "range",
				min = 0,
				max = 10,
				step = 1,
				bigStep = 1,
				set = function(info,val) self.db.global.templates[info[#info-1]].MaxBuffs = val; BUF_Factory:ApplyOptions(info[#info-1]) end,
				get = function(info) return (self.db.global.templates[info[#info-1]].MaxBuffs or 0) end
			},						
			Name = {
				order = 10,
				name = "Name",
				desc = "Options for Name",
				type = "group",
				inline = true,
				args = {
					NameJustifyV = {
						order = 0,
						name = "Vertical Alignment",
						desc = "Choose Vertical Alignment",
						type = "select",   
						confirm = false,     
						values = { ["TOP"] = "Top", ["MIDDLE"] = "Middle", ["BOTTOM"] = "Bottom", },
						get = function(info)
							return (self.db.global.templates[info[#info-2]].NameJustifyV or "TOP")
						end,
						set = function(info,val)
							self.db.global.templates[info[#info-2]].NameJustifyV = val;
							BUF_Factory:ApplyOptions(info[#info-2])
						end,						
					},									
					NameJustifyH = {
						order = 1,
						name = "Horizontal Alignment",
						desc = "Choose Horizontal Alignment",
						type = "select",   
						confirm = false,     
						values = { ["LEFT"] = "Left", ["CENTER"] = "Center", ["RIGHT"] = "Right", },
						get = function(info)
							return (self.db.global.templates[info[#info-2]].NameJustifyH or "LEFT")
						end,
						set = function(info,val)
							self.db.global.templates[info[#info-2]].NameJustifyH = val;
							BUF_Factory:ApplyOptions(info[#info-2])
						end,						
					},									
					NameFont = {
						order = 2,
						type = "select",
						dialogControl = "LSM30_Font", --Select your widget here
						name = "Name font",
						desc = "Font uses for the name text",
						values = self.LSM:HashTable(self.LSM.MediaType.FONT), -- pull in your font list from LSM
						get = function(info)
							return (self.db.global.templates[info[#info-2]].NameFont or "")
						end,
						set = function(info, val)
							self.db.global.templates[info[#info-2]].NameFont = val
							BUF_Factory:ApplyOptions(info[#info-2])
						end,
					},
					NameFontSize = {
						order = 3,
						name = "Name font size",
						desc = "Choose Name font size",					
						type = "range",
						min = 1,
						max = 20,
						step = 1,
						bigStep = 1,
						get = function(info)
							return (self.db.global.templates[info[#info-2]].NameFontSize or 11)
						end,
						set = function(info,val) 
							self.db.global.templates[info[#info-2]].NameFontSize = val
							BUF_Factory:ApplyOptions(info[#info-2])
						end,						
					},
					NameFontOutline = {
						order = 4,
						name = "Outline",
						desc = "Use outlined font",
						type = "toggle",
						get = function(info)
							return self.db.global.templates[info[#info-2]].NameFontFlags == "OUTLINE"
						end,
						set = function(info,val)
							if val then
								self.db.global.templates[info[#info-2]].NameFontFlags = "OUTLINE"
							else
								self.db.global.templates[info[#info-2]].NameFontFlags = ""
							end							
							BUF_Factory:ApplyOptions(info[#info-2])							
						end,						
					},
				},
			},
			Life = {
				order = 11,
				name = "Life",
				desc = "Options for Life",
				type = "group",
				inline = true,
				args = {
					LifeJustifyV = {
						order = 0,
						name = "Vertical Alignment",
						desc = "Choose Vertical Alignment",
						type = "select",   
						confirm = false,     
						values = { ["TOP"] = "Top", ["MIDDLE"] = "Middle", ["BOTTOM"] = "Bottom", },
						get = function(info)
							return (self.db.global.templates[info[#info-2]].LifeJustifyV or "TOP")
						end,
						set = function(info,val)
							self.db.global.templates[info[#info-2]].LifeJustifyV = val;
							BUF_Factory:ApplyOptions(info[#info-2])
						end,						
					},									
					LifeJustifyH = {
						order = 1,
						name = "Horizontal Alignment",
						desc = "Choose Horizontal Alignment",
						type = "select",   
						confirm = false,     
						values = { ["LEFT"] = "Left", ["CENTER"] = "Center", ["RIGHT"] = "Right", },
						get = function(info)
							return (self.db.global.templates[info[#info-2]].LifeJustifyH or "RIGHT")
						end,
						set = function(info,val)
							self.db.global.templates[info[#info-2]].LifeJustifyH = val;
							BUF_Factory:ApplyOptions(info[#info-2])
						end,						
					},													
					LifeFont = {
						order = 2,
						type = "select",
						dialogControl = "LSM30_Font", --Select your widget here
						name = "Name font",
						desc = "Font uses for the name text",
						values = self.LSM:HashTable(self.LSM.MediaType.FONT), -- pull in your font list from LSM
						get = function(info)
							return (self.db.global.templates[info[#info-2]].LifeFont or "")
						end,
						set = function(info, val)
							self.db.global.templates[info[#info-2]].LifeFont = val
							BUF_Factory:ApplyOptions(info[#info-2])
						end,
					},
					LifeFontSize = {
						order = 3,
						name = "Name font size",
						desc = "Choose Name font size",					
						type = "range",
						min = 1,
						max = 20,
						step = 1,
						bigStep = 1,
						get = function(info)
							return (self.db.global.templates[info[#info-2]].LifeFontSize or 11)
						end,
						set = function(info,val) 
							self.db.global.templates[info[#info-2]].LifeFontSize = val
							BUF_Factory:ApplyOptions(info[#info-2])
						end,						
					},
					LifeFontOutline = {
						order = 4,
						name = "Outline",
						desc = "Use outlined font",
						type = "toggle",
						get = function(info)
							return self.db.global.templates[info[#info-2]].LifeFontFlags == "OUTLINE"
						end,
						set = function(info,val)
							if val then
								self.db.global.templates[info[#info-2]].LifeFontFlags = "OUTLINE"
							else
								self.db.global.templates[info[#info-2]].LifeFontFlags = ""
							end							
							BUF_Factory:ApplyOptions(info[#info-2])							
						end,						
					},	
					hdr = {
						order = 5,
						name = "Misc",
						type = "header",
					},				
					LifeDisplayStyle = {
						order = 6,
						name = "Life Display Style",
						desc = "Choose Life Display Style",
						type = "select",   
						confirm = false,     
						values = { [1] = "Deficit (e.g. -2k)", [2] = "Percentage (e.g. 57%)" },
						get = function(info)
							return (self.db.global.templates[info[#info-2]].LifeDisplayStyle or 1)
						end,
						set = function(info,val)
							self.db.global.templates[info[#info-2]].LifeDisplayStyle = val;
							BUF_Factory:ApplyOptions(info[#info-2])
						end,						
					},					
				},
			},
			Incoming = {
				order = 12,
				name = "Incoming Heal",
				desc = "Options for Incoming Heal",
				type = "group",
				inline = true,
				args = {
					IncomingJustifyV = {
						order = 0,
						name = "Vertical Alignment",
						desc = "Choose Vertical Alignment",
						type = "select",   
						confirm = false,     
						values = { ["TOP"] = "Top", ["MIDDLE"] = "Middle", ["BOTTOM"] = "Bottom", },
						get = function(info)
							return (self.db.global.templates[info[#info-2]].IncomingJustifyV or "TOP")
						end,
						set = function(info,val)
							self.db.global.templates[info[#info-2]].IncomingJustifyV = val;
							BUF_Factory:ApplyOptions(info[#info-2])
						end,						
					},									
					IncomingJustifyH = {
						order = 1,
						name = "Horizontal Alignment",
						desc = "Choose Horizontal Alignment",
						type = "select",   
						confirm = false,     
						values = { ["LEFT"] = "Left", ["CENTER"] = "Center", ["RIGHT"] = "Right", },
						get = function(info)
							return (self.db.global.templates[info[#info-2]].IncomingJustifyH or "RIGHT")
						end,
						set = function(info,val)
							self.db.global.templates[info[#info-2]].IncomingJustifyH = val;
							BUF_Factory:ApplyOptions(info[#info-2])
						end,						
					},																	
					IncomingFont = {
						order = 2,
						type = "select",
						dialogControl = "LSM30_Font", --Select your widget here
						name = "Name font",
						desc = "Font uses for the name text",
						values = self.LSM:HashTable(self.LSM.MediaType.FONT), -- pull in your font list from LSM
						get = function(info)
							return (self.db.global.templates[info[#info-2]].IncomingFont or "")
						end,
						set = function(info, val)
							self.db.global.templates[info[#info-2]].IncomingFont = val
							BUF_Factory:ApplyOptions(info[#info-2])
						end,
					},
					IncomingFontSize = {
						order = 3,
						name = "Name font size",
						desc = "Choose Name font size",					
						type = "range",
						min = 1,
						max = 20,
						step = 1,
						bigStep = 1,
						get = function(info)
							return (self.db.global.templates[info[#info-2]].IncomingFontSize or 11)
						end,
						set = function(info,val) 
							self.db.global.templates[info[#info-2]].IncomingFontSize = val
							BUF_Factory:ApplyOptions(info[#info-2])
						end,						
					},
					IncomingFontOutline = {
						order = 4,
						name = "Outline",
						desc = "Use outlined font",
						type = "toggle",
						get = function(info)
							return self.db.global.templates[info[#info-2]].IncomingFontFlags == "OUTLINE"
						end,
						set = function(info,val)
							if val then
								self.db.global.templates[info[#info-2]].IncomingFontFlags = "OUTLINE"
							else
								self.db.global.templates[info[#info-2]].IncomingFontFlags = ""
							end
							BUF_Factory:ApplyOptions(info[#info-2])							
						end,						
					},				
					IncomingBarColor = {
						order = 5,
						name = "Bar color",
						desc = "Define the incoming heal bar color",
						type = "color",
						hasAlpha = true,
						get = function(info)
							local tpl = self.db.global.templates[info[#info-2]]
							return tpl.IncomingBarR, tpl.IncomingBarG, tpl.IncomingBarB, tpl.IncomingBarA
						end,
						set = function (info, r, g, b, a)							
							local tpl = self.db.global.templates[info[#info-2]]
							tpl.IncomingBarR = r
							tpl.IncomingBarG = g
							tpl.IncomingBarB = b
							tpl.IncomingBarA = a
							BUF_Factory:ApplyOptions(info[#info-2])
						end
					},
				},								
			},						
		},
	}		
end

local function GetClickcastSpell(info)
	local currentBtn = info[#info]
	local className = info[#info - 1]
	
	return bUF.db.global.clickcast[className][currentBtn]
end

local function SetClickcastSpell(info, val)
	local currentBtn = info[#info]
	local className = info[#info - 1]

	bUF.db.global.clickcast[className][currentBtn] = (GetSpellInfo(val))
	BUF_Roster:UpdateAllSecureButton()
end

local function ValidateClickcastSpell(info, val)
	local name = GetSpellInfo(val)
		
	if not name then
		return "Unknown Spellname or SpellId"
	else
		return true
	end
end

function bUF:ClickcastOptions(className)
	self.options.args.clickcast.args[className] = {
		name = LOCALIZED_CLASS_NAMES_MALE[className],
		desc = "Spell settings for " .. LOCALIZED_CLASS_NAMES_MALE[className],
		type = "group",
		args = {
			spell2 = {
				name = "Right Button",
				desc = "Enter Spellname or SpellId for Right Button",
				type = "input",   
				confirm = false,
				validate = ValidateClickcastSpell,
				set = SetClickcastSpell,
				get = GetClickcastSpell,
			},			
			spell3 = {
				name = "Middle Button",
				desc = "Enter Spellname or SpellId for Middle Button",
				type = "input",   
				confirm = false,     
				validate = ValidateClickcastSpell,
				set = SetClickcastSpell,
				get = GetClickcastSpell,
			},
			spell4 = {
				name = "Button 4",
				desc = "Enter Spellname or SpellId for Button 4",
				type = "input",   
				confirm = false,     
				validate = ValidateClickcastSpell,
				set = SetClickcastSpell,
				get = GetClickcastSpell,
			},
			spell5 = {
				name = "Button 5",
				desc = "Enter Spellname or SpellId for Button 5",
				type = "input",   
				confirm = false,     
				validate = ValidateClickcastSpell,
				set = SetClickcastSpell,
				get = GetClickcastSpell,
			},
		},
	}
end

local function GetBuffSpell(info)
	local currentBtn = info[#info]
	local className = info[#info - 1]
	
	return bUF.db.global.buffs[className][currentBtn]
end

local function SetBuffSpell(info, val)
	local currentBtn = info[#info]
	local className = info[#info - 1]

	if val == nil or val == "" then
		bUF.db.global.buffs[className][currentBtn] = nil
	else
		if tonumber(val) then
			bUF.db.global.buffs[className][currentBtn] = val
		else
			bUF.db.global.buffs[className][currentBtn] = (GetSpellInfo(val))
		end
	end
	BUF_Roster:UpdateUnitFrameStyle()
end

local function ValidateBuffSpell(info, val)
	if val == nil or val == "" then return true end
	local name = GetSpellInfo(val)
		
	if not name then
		return "Unknown Spellname or SpellId"
	else
		return true
	end
end

function bUF:BuffOptions(className)
	self.options.args.buffs.args[className] = {
		name = LOCALIZED_CLASS_NAMES_MALE[className],
		desc = "Buff settings for " .. LOCALIZED_CLASS_NAMES_MALE[className],
		type = "group",
		args = {
			buff1 = {
				name = "Buff 1",
				desc = "Enter Buffname or SpellId for Buff Slot 1",
				type = "input",   
				confirm = false,
				validate = ValidateBuffSpell,
				set = SetBuffSpell,
				get = GetBuffSpell,
			},			
			buff2 = {
				name = "Buff 2",
				desc = "Enter Buffname or SpellId for Buff Slot 2",
				type = "input",   
				confirm = false,     
				validate = ValidateBuffSpell,
				set = SetBuffSpell,
				get = GetBuffSpell,
			},
			buff3 = {
				name = "Buff 3",
				desc = "Enter Buffname or SpellId for Buff Slot 3",
				type = "input",   
				confirm = false,     
				validate = ValidateBuffSpell,
				set = SetBuffSpell,
				get = GetBuffSpell,
			},
			buff4 = {
				name = "Buff 4",
				desc = "Enter Buffname or SpellId for Buff Slot 4",
				type = "input",   
				confirm = false,     
				validate = ValidateBuffSpell,
				set = SetBuffSpell,
				get = GetBuffSpell,
			},
		},
	}
end

function bUF:IncomingHealOptions()
	self.options.args.IncomingHeal = {
		name = "Incoming Heal",
		desc = "Configure Incoming Heal options",
		type = "group",
		args = {
			Enabled = {		
				name = "Enabled",
				desc = "Show incoming heal",
				type = "toggle",
				set = function(info,val) self.db.global.IncomingHeal = val; self:ApplyIncomingHeal() end,
				get = function(info) return self.db.global.IncomingHeal end
			},
			OnlyOthers = {
				name = "Only other's",
				desc = "Show only incoming heal from other players",
				type = "toggle",
				set = function(info,val) self.db.global.IncomingHealOnlyOthers = val; end,
				get = function(info) return self.db.global.IncomingHealOnlyOthers end					
			},
			OnlyDirect = {
				name = "Only direct heal",
				desc = "Show only direct heal casts (e.g. Flash Heal)",
				type = "toggle",
				set = function(info,val) self.db.global.IncomingHealOnlyDirect = val; end,
				get = function(info) return self.db.global.IncomingHealOnlyDirect end					
			},
			TimeWindow = {
				name = "Time Window",
				desc = "Only consider incoming heal from next seconds",
				type = "range",
				min = 1,
				max = 10,
				step = 1,
				bigStep = 1,
				set = function(info,val) self.db.global.IncomingHealTimeWindow = val end,
				get = function(info) return (self.db.global.IncomingHealTimeWindow or 1) end
			},
		},
	}
end

function bUF:BuffCheckOptions()
	self.options.args.BuffCheck = {
		name = "Buff Check",
		desc = "Configure Buff Check options",
		type = "group",
		args = {
			CheckFlasks = {		
				name = "Flasks",
				desc = "Check for missing flasks",
				type = "toggle",
				set = function(info,val) self.db.global.CheckFlasks = val end,
				get = function(info) return self.db.global.CheckFlasks end
			},
			CheckFood = {
				name = "Food",
				desc = "Check for missing food buff",
				type = "toggle",
				set = function(info,val) self.db.global.CheckFood = val end,
				get = function(info) return self.db.global.CheckFood end					
			},
			AnnounceFeast = {
				name = "Announce Feast",
				desc = "Announce the preparation of a fish feast (requires L/A)",
				type = "toggle",
				set = function(info,val) self.db.global.AnnounceFeast = val end,
				get = function(info) return self.db.global.AnnounceFeast end					
			},
		},
	}
end

function bUF:AuraOptions()
	self.options.args.Aura = {
		name = "Aura",
		desc = "Configure Aura options",
		type = "group",
		args = {
			Count = {
				order = 1,
				name = "Stack Count",
				desc = "Options for Stack Count",
				type = "group",
				inline = true,
				args = {
					CountFont = {
						order = 2,
						type = "select",
						dialogControl = "LSM30_Font", --Select your widget here
						name = "Count font",
						desc = "Font uses for the Count text",
						values = self.LSM:HashTable(self.LSM.MediaType.FONT), -- pull in your font list from LSM
						get = function(info)
							return (self.db.global.aura.CountFont or "")
						end,
						set = function(info, val)
							self.db.global.aura.CountFont = val
							BAF_Factory:ApplyOptions(self.db.global.aura)
							BUF_Roster:UpdateUnitFrameStyle()
						end,
					},
					CountFontSize = {
						order = 3,
						name = "Count font size",
						desc = "Choose Count font size",					
						type = "range",
						min = 1,
						max = 20,
						step = 1,
						bigStep = 1,
						get = function(info)
							return (self.db.global.aura.CountFontSize or 11)
						end,
						set = function(info,val) 
							self.db.global.aura.CountFontSize = val
							BAF_Factory:ApplyOptions(self.db.global.aura)
							BUF_Roster:UpdateUnitFrameStyle()
						end,						
					},
					CountFontOutline = {
						order = 4,
						name = "Outline",
						desc = "Use outlined font",
						type = "toggle",
						get = function(info)
							return self.db.global.aura.CountFontFlags == "OUTLINE"
						end,
						set = function(info,val)
							if val then
								self.db.global.aura.CountFontFlags = "OUTLINE"
							else
								self.db.global.aura.CountFontFlags = ""
							end
							BAF_Factory:ApplyOptions(self.db.global.aura)
							BUF_Roster:UpdateUnitFrameStyle()					
						end,						
					},				
					CountFontColor = {
						order = 5,
						name = "Count Font Color",
						desc = "Define the Count font color",
						type = "color",
						hasAlpha = true,
						get = function(info)
							local tpl = self.db.global.aura
							return tpl.CountFontR, tpl.CountFontG, tpl.CountFontB, tpl.CountFontA
						end,
						set = function (info, r, g, b, a)							
							local tpl = self.db.global.aura
							tpl.CountFontR = r
							tpl.CountFontG = g
							tpl.CountFontB = b
							tpl.CountFontA = a
							BAF_Factory:ApplyOptions(self.db.global.aura)
							BUF_Roster:UpdateUnitFrameStyle()
						end
					},		
				},
			},
			Cowndown = {
				order = 1,
				name = "Cowndown",
				desc = "Options for Cowndown",
				type = "group",
				inline = true,
				args = {
					CowndownFont = {
						order = 2,
						type = "select",
						dialogControl = "LSM30_Font", --Select your widget here
						name = "Cowndown font",
						desc = "Font uses for the Cowndown text",
						values = self.LSM:HashTable(self.LSM.MediaType.FONT), -- pull in your font list from LSM
						get = function(info)
							return (self.db.global.aura.CowndownFont or "")
						end,
						set = function(info, val)
							self.db.global.aura.CowndownFont = val
							BAF_Factory:ApplyOptions(self.db.global.aura)
							BUF_Roster:UpdateUnitFrameStyle()
						end,
					},
					CowndownFontSize = {
						order = 3,
						name = "Cowndown font size",
						desc = "Choose Cowndown font size",					
						type = "range",
						min = 1,
						max = 20,
						step = 1,
						bigStep = 1,
						get = function(info)
							return (self.db.global.aura.CowndownFontSize or 11)
						end,
						set = function(info,val) 
							self.db.global.aura.CowndownFontSize = val
							BAF_Factory:ApplyOptions(self.db.global.aura)
							BUF_Roster:UpdateUnitFrameStyle()
						end,						
					},
					CowndownFontOutline = {
						order = 4,
						name = "Outline",
						desc = "Use outlined font",
						type = "toggle",
						get = function(info)
							return self.db.global.aura.CowndownFontFlags == "OUTLINE"
						end,
						set = function(info,val)
							if val then
								self.db.global.aura.CowndownFontFlags = "OUTLINE"
							else
								self.db.global.aura.CowndownFontFlags = ""
							end
							BAF_Factory:ApplyOptions(self.db.global.aura)	
							BUF_Roster:UpdateUnitFrameStyle()					
						end,						
					},				
					CowndownFontColor = {
						order = 5,
						name = "Cowndown Font Color",
						desc = "Define the Cowndown font color",
						type = "color",
						hasAlpha = true,
						get = function(info)
							local tpl = self.db.global.aura
							return tpl.CowndownFontR, tpl.CowndownFontG, tpl.CowndownFontB, tpl.CowndownFontA
						end,
						set = function (info, r, g, b, a)							
							local tpl = self.db.global.aura
							tpl.CowndownFontR = r
							tpl.CowndownFontG = g
							tpl.CowndownFontB = b
							tpl.CowndownFontA = a
							BAF_Factory:ApplyOptions(self.db.global.aura)
							BUF_Roster:UpdateUnitFrameStyle()
						end
					},		
				},
			},
		},
	}
end

function BUF_Roster:InsertOptions(options, area)
	options.args.areas.args[area.Name] = {
	    name = area.Name,
		type = "group",
		args = {
			Enabled = {
				name = "Enabled",
				desc = "Area enabled",
				type = "toggle",
				set = function(info,val) self.db.global.area[info[#info-1]].Enabled = val; BUF_Roster:RearrangeAll() end,
				get = function(info) return self.db.global.area[info[#info-1]].Enabled end
			},
			Spacer = {      
				name = "Spacer",
				desc = "Size of spacer between units",
				type = "range",
				min = 0,
				max = 20,
				step = 1,
				bigStep = 1,
				set = function(info,val) self.db.global.area[info[#info-1]].Spacer = val; BUF_Roster:RearrangeAll() end,
				get = function(info) return self.db.global.area[info[#info-1]].Spacer end
			},
			MaxColumns = {      
				name = "Columns",
				desc = "Maximal count of columns",
				type = "range",
				min = 1,
				max = 10,
				step = 1,
				bigStep = 1,
				set = function(info,val) self.db.global.area[info[#info-1]].MaxColumns = val; BUF_Roster:RearrangeAll() end,
				get = function(info) return self.db.global.area[info[#info-1]].MaxColumns end
			},			
			MaxRows = {      
				name = "Rows",
				desc = "Maximal count of rows",
				type = "range",
				min = 1,
				max = 10,
				step = 1,
				bigStep = 1,
				set = function(info,val) self.db.global.area[info[#info-1]].MaxRows = val; BUF_Roster:RearrangeAll() end,
				get = function(info) return self.db.global.area[info[#info-1]].MaxRows end
			},			
			MinDisplayRows = {      
				name = "Minimal Rows",
				desc = "Minimal count of rows used for height calculation",
				type = "range",
				min = 1,
				max = 10,
				step = 1,
				bigStep = 1,
				set = function(info,val) self.db.global.area[info[#info-1]].MinDisplayRows = val; BUF_Roster:RearrangeAll() end,
				get = function(info) return self.db.global.area[info[#info-1]].MinDisplayRows end
			},	
			Horizontal = {
				name = "Horizontal",
				desc = "Horizontal party layout",
				type = "toggle",
				set = function(info,val) self.db.global.area[info[#info-1]].Horizontal = val; BUF_Roster:RearrangeAll() end,
				get = function(info) return self.db.global.area[info[#info-1]].Horizontal end
			},	
			Template = {
				name = "Template",
				desc = "Choose Area Template",
				type = "select",   
				confirm = false,     
				values = function(info)
					local vals = {}
					for k, _ in pairs(BUF_Factory.templates) do
						vals[k] = k
					end
					return vals
				end,
				get = function(info)
					return self.db.global.area[info[#info-1]].Template or "standard"
				end,
				set = function(info,val)
					self.db.global.area[info[#info-1]].Template = val
					BUF_Roster:RearrangeAll() 
				end,						
			},				
		},
    }	  
end